﻿using Revision.Models;
using Revision.Models.ViewModels;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;

namespace Revision.Controllers
{
    public class TestController : BaseController
    {
        /// <summary>
        /// The current exam being taken, stored in Session
        /// </summary>
        private Exam CurrentExam
        {
            get { return (Exam)Session["CurrentExam"]; }
            set { Session["CurrentExam"] = value; }
        }

        //
        // GET: /Test/
        // Starts a new exam
        [HttpGet]
        public ActionResult Index(string examcode, string section = "")
        {
            // Choose the exam
            SetCurrentExam(examcode, section);

            if (CurrentExam.Questions.Count == 0)
                return RedirectToAction("Index", "Home");

            // Start with the first question
            return RedirectToAction("Question", new { id = 1 });
        }

        // Changes the exam being taken, to the one with this examcode
        private void SetCurrentExam(string examcode, string section)
        {
            CurrentExam = null;

            // Load exam 
            Exam exam = GetExam(examcode, IsRedacted());

            if (exam == null)
                throw new ApplicationException("Unknown exam: " + examcode);

            // Only this section
            if (!string.IsNullOrWhiteSpace(section))
                exam.FilterQuestions(section);

            // Reset the exam
            exam.Clear();

            CurrentExam = exam;
        }


        // Returns the exam for this examcode, or null
        private Exam GetExam(string examcode, bool isTextRedacted)
        {
            switch (examcode)
            {
                case "70-486":
                    return new ExamMVC4(isTextRedacted);

                case "70-487":
                    return new ExamAzure(isTextRedacted);
            
                case "70-483":
                    return new ExamCSharp(isTextRedacted);
            
                default:
                    return null;
            }
        }


        // Displays the Nth question
        [HttpGet]
        public ActionResult Question(int id)
        {
            QuestionView model = new QuestionView(id, CurrentExam);

            return View(model);
        }

        // Saves the answers to this question, then on to the next one
        [HttpPost]
        [ValidateInput(false)]
        public ActionResult Question(int id, 
                                     bool[] checkboxes,
                                     string[] textboxes,
                                     string nav)
        {
            // Save the answers to the current question
            Exam exam = CurrentExam;
            Question question = exam.NthQuestion(id);

            question.UpdateAnswers(checkboxes, textboxes);

            CurrentExam = exam;

            // Get next page number
            int nextid = (nav == "Next" ? id + 1 : id - 1);

            return RedirectToNextQuestion(nextid);
        }

        // Redirects to the next question, or "End of exam"
        private RedirectToRouteResult RedirectToNextQuestion(int id)
        {
            Exam exam = CurrentExam;

            if (id <= 0)
                return RedirectToAction("Question", new { id = 1 });

            if (id > exam.Questions.Count)
                return RedirectToAction("Completed");

            return RedirectToAction("Question", new { id = id });
        }

        // Displays the completed exam results
        [HttpGet]
        public ActionResult Completed()
        {
            return View(CurrentExam);
        }

        //////////////// Marked Answers ////////////////

        // Reviews the marked answers
        [HttpGet]
        public ActionResult Answer(int id, bool incorrectonly = false)
        {
            // Get the Nth question (or Nth incorrect question)
            Exam exam = CurrentExam;

            Question question = (incorrectonly
                                    ? exam.NthIncorrectlyAnswered(id)
                                    : exam.NthQuestion(id));

            if (question == null)
                return RedirectToAction("Completed");

            // View the answers to that question
            QuestionView model = new QuestionView(id, exam, question, incorrectonly);

            return View(model); 
        }

        // Moves on to the next answer
        [HttpPost]
        public ActionResult Answer(int id, string nav, bool incorrectonly = false)
        {
            int nextid = (nav == "Next" ? id + 1 : id - 1);

            return RedirectToAction("Answer", new { id = nextid, incorrectonly = incorrectonly });
        }

        ////////////// Exam Tips ///////////////////

        [HttpGet]
        public ActionResult Tips(string examcode, string section = "")
        {
            // Choose the exam
            SetCurrentExam(examcode, section);

            Exam exam = CurrentExam;

            if (exam.ExamTips.Count == 0)
                return RedirectToAction("Index", "Home");

            // Display the hints
            TipView model = new TipView(exam, section);

            return View(model);
        }
    }
}
