﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using Revision.Utilities;

namespace Revision.Models
{
    /// <summary>
    /// Set of questions, used to revise for an Exam
    /// </summary>
    public abstract class Exam : BaseQA
    {
        /// <summary>
        /// Set of questions, used to revise for an Exam
        /// </summary>
        public Exam(string title, string examCode, int passMark)
        {
            Questions = new List<Question>();
            ExamTips  = new List<Tip>();
            Title = title;
            ExamCode = examCode;
            PassMark = passMark;
        }

        /// <summary>
        /// Name of the exam
        /// </summary>
        public string Title { get; private set; }

        /// <summary>
        /// Unique exam code
        /// </summary>
        public string ExamCode { get; private set; }

        /// <summary>
        /// Get this mark or better to pass, out of 100
        /// </summary>
        public int PassMark { get; private set; }

        /// <summary>
        /// All questions in the exam
        /// </summary>
        public List<Question> Questions { get; set; }

        /// <summary>
        /// All tips for this exam
        /// </summary>
        public List<Tip> ExamTips { get; set; }

        /// <summary>
        /// Current background/context, added to each new question
        /// </summary>
        protected string CurrentBackground;

        /// <summary>
        /// Randomly sorts all question and answers, and clears any previous answers
        /// </summary>
        public void Clear()
        {
            Questions.Shuffle();

            foreach (Question q in Questions)
            {
                q.Clear();
            }
        }

        /////////////////////// Checkbox questions & answers ////////////////////

        /// <summary>
        /// Adds a multiple-choice question to the exam.
        /// </summary>
        public void MC(string questionNumber, string questionText, CheckboxAnswer[] possibleAnswers)
        {
            AddQuestion(questionNumber, 
                        questionText, 
                        possibleAnswers, 
                        isSingleAnswer: false);
        }

        /// <summary>
        /// Adds a multiple-choice "source code" question to the exam.
        /// </summary>
        public void MCC(string questionNumber, string questionText, CheckboxAnswer[] possibleAnswers)
        {
            AddQuestion(questionNumber,
                        questionText,
                        possibleAnswers,
                        isSingleAnswer: false,
                        isSourceCodeQuestion: true,
                        isSourceCodeAnswer: true);
        }

        /// <summary>
        /// Adds a single-choice question to the exam.
        /// </summary>
        public void SC(string questionNumber, string questionText, CheckboxAnswer[] possibleAnswers)
        {
            AddQuestion(questionNumber, 
                        questionText, 
                        possibleAnswers, 
                        isSingleAnswer: true);
        }

        /// <summary>
        /// Adds a single-choice "code" question to the exam.
        /// </summary>
        public void SCC(string questionNumber, string questionText, CheckboxAnswer[] possibleAnswers)
        {
            AddQuestion(questionNumber, 
                        questionText, 
                        possibleAnswers, 
                        isSingleAnswer: true,
                        isSourceCodeQuestion: true,
                        isSourceCodeAnswer: true);
        }

        /////////////////////// Checkbox questions & answers ////////////////////

        /// <summary>
        /// Adds a multiple-textbox question (aka "thought experiment") to the exam
        /// </summary>
        public void MT(string questionNumber, 
                       string background,
                       string questionText, 
                       TextboxAnswer[] textboxAnswers,
                       bool isSourceCodeQuestion = false)
        {
            AddQuestion(questionNumber,
                        questionText,
                        textboxAnswers,
                        isSingleAnswer: false,
                        displayInstructions: false,
                        background: background,
                        isSourceCodeQuestion: isSourceCodeQuestion,
                        isSourceCodeAnswer: false,
                        sortableAnswers: false);
        }


        /// <summary>
        /// Adds a formatted multiple-textbox question (aka "thought experiment") to the exam
        /// </summary>
        public void MTC(string questionNumber, 
                        string background,
                        string questionText, 
                        TextboxAnswer[] textboxAnswers)
        {
            MT(questionNumber,
               background,
               questionText,
               textboxAnswers,
               isSourceCodeQuestion: true);
        }


        /////////////////////// Adding questions ////////////////////////

        // Adds a question to the exam
        private void AddQuestion(string questionNumber, 
                                 string questionText, 
                                 IPossibleAnswer[] possibleAnswers, 
                                 bool   isSingleAnswer,
                                 bool   isSourceCodeQuestion = false,
                                 bool   isSourceCodeAnswer   = false,
                                 bool   displayInstructions  = true,
                                 string background           = null,
                                 bool   sortableAnswers      = true)
        {
            if (string.IsNullOrWhiteSpace(questionText))
                return;

            CheckAnswerFormat(questionNumber, 
                              isSingleAnswer, 
                              possibleAnswers);

            Question q = new Question()
            {
                QuestionNumber      = questionNumber,
                IsSingleAnswer      = isSingleAnswer,
                IsSourceCode        = isSourceCodeQuestion,
                Background          = ProcessText(background ?? CurrentBackground),
                QuestionText        = ProcessText(questionText),
                Answers             = possibleAnswers,
                DisplayInstructions = displayInstructions,
                SortableAnswers     = sortableAnswers,
                IsTextRedacted      = this.IsTextRedacted
            };

            foreach (IPossibleAnswer a in q.Answers)
            {
                // Source code must be specially formatted
                if (isSourceCodeAnswer)
                    a.IsSourceCode = isSourceCodeAnswer;

                // Redact answers and explanations, as well as questions
                a.IsTextRedacted = this.IsTextRedacted;

                a.AnswerText = ProcessText(a.AnswerText);
                a.Explanation = ProcessText(a.Explanation);
            }

            Questions.Add(q);
        }

        // Checks the answers, and throws an exception if they violate the question format
        private void CheckAnswerFormat(string questionNumber,
                                       bool isSingleAnswer, 
                                       IPossibleAnswer[] possibleAnswers)
        {
            // Unmarkable answers cannot violate the format
            bool isMarkable = possibleAnswers.Where(a => a.IsMarkable).Any();

            if (!isMarkable)
                return;

            // Single-answer questions can only have one correct answer
            int correctCount = possibleAnswers.Where(a => a.IsCorrect && a.IsMarkable)
                                              .Count();

            if (isSingleAnswer && correctCount != 1)
                ThrowAnswerCount(questionNumber, correctCount);

            // Every question must have at least one correct answer
            if (correctCount == 0)
                ThrowAnswerCount(questionNumber, correctCount);
        }

        // Throws an exception because this question has the wrong number of correct answers
        private void ThrowAnswerCount(string questionNumber, int correctCount)
        {
                throw new ApplicationException("Defintion for question '" 
                                               + questionNumber 
                                               + "' has " 
                                               + correctCount.ToString() 
                                               + " correct answers.");
        }


        /// <summary>
        /// Removes all questions, except with this prefix for QuestionName
        /// </summary>
        public void FilterQuestions(string questionNamePrefix)
        {
            Questions = Questions.Where(q => q.QuestionNumber.StartsWith(questionNamePrefix, 
                                                                         StringComparison.OrdinalIgnoreCase))
                                 .ToList();
        }

        //////////////// Exam Tips ////////////////

        /// <summary>
        /// Adds an Exam Tip, using the current background.
        /// </summary>
        public void ET(string sectionNumber, string heading, string text)
        {
            if (string.IsNullOrWhiteSpace(heading) ||
                string.IsNullOrWhiteSpace(text))
                return;

            // Valid tip - add it
            Tip t = new Tip()
            {
                Background    = ProcessText(CurrentBackground),
                SectionNumber = sectionNumber,
                Heading       = ProcessText(heading),
                Text          = ProcessText(text)
            };

            ExamTips.Add(t);
        }

        //////////////// Navigation ///////////////

        /// <summary>
        /// Returns the Nth question in this exam, or null
        /// </summary>
        public Question NthQuestion(int n)
        {
            return Questions.Nth(n);
        }

        /// <summary>
        /// Returns the Nth incorrectly answered question in this exam, or null
        /// </summary>
        public Question NthIncorrectlyAnswered(int n)
        {
            return AnsweredIncorrectlyOrUnmarked.Nth(n);
        }

        ///////////// Results ////////////////

        /// <summary>
        /// All questions which have been answered correctly
        /// </summary>
        private IEnumerable<Question> AnsweredCorrectly
        {
            get 
            {
                return Questions.Where(q => q.IsMarkable && q.IsCorrect);
            }
        }

        /// <summary>
        /// All questions which have been answered wrongly
        /// </summary>
        private IEnumerable<Question> AnsweredIncorrectly
        {
            get
            {
                return Questions.Where(q => q.IsMarkable && 
                                            !q.IsCorrect &&
                                            !q.IsUnanswered);
            }
        }

        /// <summary>
        /// All questions which have been answered wrongly, or uncertain
        /// </summary>
        public IEnumerable<Question> AnsweredIncorrectlyOrUnmarked
        {
            get
            {
                return Questions.Where(q => !q.IsMarkable || (!q.IsCorrect && 
                                                              !q.IsUnanswered));
            }
        }

        /// <summary>
        /// Number of questions answered correctly
        /// </summary>
        public int MarkedCorrect
        { 
            get { return AnsweredCorrectly.Count(); } 
        }

        /// <summary>
        /// Number of questions answered wrongly
        /// </summary>
        public int MarkedIncorrect
        {
            get { return AnsweredIncorrectly.Count(); }
        }

        /// <summary>
        /// Total number of markable questions in exam
        /// </summary>
        public int MarkedCount 
        {
            get { return Questions.Count(q => q.IsMarkable);  }
        }

        /// <summary>
        /// Number of unanswered questions
        /// </summary>
        public int MarkedSkipped 
        {
            get { return MarkedCount - (MarkedCorrect + MarkedIncorrect); }
        }

        /// <summary>
        /// Percentage of questions which were answered correctly
        /// </summary>
        public int MarkedPercentage 
        {
            get 
            {
                if (MarkedCount == 0)
                    return 0;

                return (int)(100.0 * MarkedCorrect / MarkedCount); 
            }
        }

        /// <summary>
        /// Did the user pass the exam?
        /// </summary>
        public bool Passed
        {
            get { return MarkedPercentage >= PassMark; }
        }
    }
}