﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using Revision.Utilities;

namespace Revision.Models
{
    /// <summary>
    /// Multiple-choice question (many possible answers)
    /// </summary>
    public class Question : BaseQA
    {
        /// <summary>
        /// Displayable section and question number eg. "1.4 (1)"
        /// </summary>
        public string QuestionNumber { get; set; }

        /// <summary>
        /// Background information on the question
        /// </summary>
        public string Background { get; set; }

        /// <summary>
        /// The question being asked
        /// </summary>
        public string QuestionText { get; set; }

        /// <summary>
        /// Is this a single-answer question?
        /// </summary>
        public bool IsSingleAnswer { get; set; }

        /// <summary>
        /// Does the user need instructions on how to answer the question?
        /// </summary>
        public bool DisplayInstructions { get; set; }

        /// <summary>
        /// Do the answers to this question contain source code? 
        /// (which needs to be formatted exactly as-is)
        /// </summary>
        public bool IsSourceCode { get; set; }

        /// <summary>
        /// All possible answers
        /// </summary>
        public IPossibleAnswer[] Answers { get; set; }

        /// <summary>
        /// Can the answers be sorted, and still make sense?
        /// </summary>
        public bool SortableAnswers { get; set; }

        /// <summary>
        /// Did the user answer this question correctly?
        /// </summary>
        public bool IsCorrect
        {
            get 
            { 
                return Answers.All(a => a.IsMarkable && a.IsAnsweredCorrectly); 
            }
        }

        /// <summary>
        /// Did the user skip this question?
        /// </summary>
        public bool IsUnanswered
        {
            get { return !(Answers.Any(a => a.IsAnswered)); }
        }

        /// <summary>
        /// Can this question be marked?
        /// </summary>
        public bool IsMarkable
        {
            get { return Answers.Any(a => a.IsMarkable); }
        }

        /// <summary>
        /// Resets the question, clearing and re-ordering the answers
        /// </summary>
        public void Clear()
        {
            if (SortableAnswers)
                Answers.Shuffle();

            foreach (IPossibleAnswer a in Answers)
            {
                a.Clear();
            }
        }

        /// <summary>
        /// Sets all Answers.ChosenAnswers 
        /// </summary>
        public void UpdateAnswers(bool[] checkboxes, string[] textboxes)
        {
            for (int i = 0; i < Answers.Length; ++i)
            {
                if (checkboxes != null)
                    Answers[i].ChosenAnswer = checkboxes[i];

                if (textboxes != null)
                    Answers[i].ChosenText   = textboxes[i];
            }
        }
    }
}