﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;

namespace Revision.Models.ViewModels
{
    /// <summary>
    /// Asks a question, and posts the answer back to the QuestionController.
    /// Viewing model for 
    ///     Views/Test/Question.cshtml
    ///     Views/Test/Answer.cshtml
    /// </summary>
    public class QuestionView
    {
        /// <summary>
        /// Question number, 1-based
        /// </summary>
        public int Number { get; set; }

        /// <summary>
        /// Total number of questions in this exam
        /// </summary>
        public int TotalNumber { get; set; }

        /// <summary>
        /// The question being asked, together with the answers
        /// </summary>
        public Question ThisQuestion { get; set; }

        /// <summary>
        /// The examination being taken
        /// </summary>
        public Exam ThisExam { get; set; }

        /// <summary>
        /// Ticked answers, corresponding to Data.Answers. 
        /// For checkboxes, returned to the controller.
        /// </summary>
        public bool[] Checkboxes { get; set; }

        /// <summary>
        /// Text answers, corresponding to Data.Answers. 
        /// Returned to the controller for unmarkable answers.
        /// </summary>
        [AllowHtml]
        public string[] Textboxes { get; set; }

        //////////// Display //////////////

        /// <summary>
        /// Various display modes for the questions and answers
        /// </summary>
        public enum DisplayMode 
        {
            Questions,
            AllAnswers,
            IncorrectAnswers
        }

        /// <summary>
        /// Viewing mode (Questions, AllAnswers, IncorrectAnswers)
        /// </summary>
        public DisplayMode Mode { get; private set; }

        ///////////// Constructors /////////////

        public QuestionView() {}

        /// <summary>
        /// Asks a question, and posts the answer back to the QuestionController.
        /// Viewing model for Views/Test/Question.cshtml
        /// </summary>
        public QuestionView(int number, Exam exam)
        {
            Number       = number;
            ThisExam     = exam;
            ThisQuestion = exam.NthQuestion(number);

            Mode         = DisplayMode.Questions;
            TotalNumber  = exam.Questions.Count;

            InitAnswers();
        }

        /// <summary>
        /// Reviews the answers to a question.
        /// Viewing model for Views/Test/Answer.cshtml
        /// </summary>
        public QuestionView(int number, Exam exam, Question question, bool incorrectOnly)
        {
            Number       = number;
            ThisExam     = exam;
            ThisQuestion = question;

            if (incorrectOnly)
            {
                Mode        = DisplayMode.IncorrectAnswers;
                TotalNumber = exam.AnsweredIncorrectlyOrUnmarked.Count();
            }
            else
            {
                Mode        = DisplayMode.AllAnswers;
                TotalNumber = exam.Questions.Count;
            }

            InitAnswers();
        }

        // Sets up the Answers 
        private void InitAnswers()
        {
            Checkboxes = ThisQuestion.Answers.ToList()
                                             .ConvertAll(a => a.ChosenAnswer)
                                             .ToArray();

            Textboxes = ThisQuestion.Answers.ToList()
                                            .ConvertAll(a => a.ChosenText)
                                            .ToArray();
        }
    }
}